//-----------------------------------------------------------------------------
/*
    CGame.h - Author: Marcus Krner - 2003 - This source is GPL
    class providing our game
    needs a CSDL, CIsoEngine and a CLog instance
*/

#ifndef CGAME_H_
#define CGAME_H_

#include <iostream>
#include <string>
#include <sstream>
#include <SDL/SDL.h>

#include "CSDL.h"
#include "CSimpleFont.h"
#include "CIsoEngine.h"
#include "CLog.h"
#include "CEdit.h"
#include "menu.h"

// Gameentities
#include "entities/CEntity.h"
#include "entities/CTower.h"
#include "entities/CEnvironment.h"

// particlestuff
#include "particle/particleeffect.h"
#include "particle/imageparticleeffect.h"

// chars
#include "chars/char.h"
#include "chars/simplechar.h"

//attacks
#include "attacks/attack.h"

using namespace std;

//-----------------------------------------------------------------------------
/*
    class CGame - our game class
*/
class CGame
{
    CSDL* m_pSDLFrame;
    CIsoEngine* m_pEngine;
    CLog* m_pLog;
    CEdit* m_pEdit;
    CSimpleFont *m_pFont;
    Menu *m_pMenu;

    //testing particle effect
  ParticleEffect *m_pEffect1;
  ImageParticleEffect *m_pEffect2;

    Char *char1, *char2;
    Attack *attack1, *attack2;
    
            
    CEntity** m_pPrototype;                    // prototypes for entities
    CEntity* m_pEntitylist;                        // list with all current entities
    CEntity** m_pMapEntity;                   // Map with entitiyreferences on it

    Char* m_pCharlist;                        // list with all current entities
    Char** m_pMapChar;                   // Map with entitiyreferences on it
    
    int m_iPrototypeNumber;                   // existing prototypes overall
    int m_iActEntities;                                // entities in game
    int m_iActChars;                                // entities in game
    int m_iTILEHEIGHT;                            // height of the groundtile
    int m_iTILEWIDTH;                              // width of the groundtile
    
    unsigned long m_dwFrameStart;
    unsigned long m_dwFrameTime;

    bool m_bEditMode;                                // take out for release

public:

    CGame();
    ~CGame();

    CSDL* getSDLFrame(){ return m_pSDLFrame; };
    
    CEntity* getEntitylist(){ return m_pEntitylist; };
    CEntity** getPrototype(){ return m_pPrototype; };
    CEntity** getMapEntity(){ return m_pMapEntity; };

    bool idle();                                                // gameloop
    bool pickEntity();

    void init( CSDL *sdlFrame, CIsoEngine *engine, CLog *log );
    void loadPrototypes( string file );
    void addEntity( CEntity *entity );
    void addChar( Char *c );
    void deleteEntity( CEntity *entity );
    void deleteChar( Char *c );
    void drawEntities();
    void unselectChars();
    void enableEntityDraw();
};

#endif
