//-----------------------------------------------------------------------------
/*
    CIsoEngine.cpp - Author: Marcus Krner - 2003 - This source is GPL
*/

#include "CIsoEngine.h"

//-----------------------------------------------------------------------------
CIsoEngine::CIsoEngine()
{
}

//-----------------------------------------------------------------------------
CIsoEngine::~CIsoEngine()
{
    delete[] m_pMap;
    delete[] m_pGround;
    delete[] m_pDetail;
}

//-----------------------------------------------------------------------------
/*
	set up the whole iso engine
*/
bool CIsoEngine::init( CSDL *sdlFrame, CLog *log, string file )
{
    m_pSDLFrame = sdlFrame;
    m_pLog = log;
    m_pLog->write( "|" );
    
    // set scrolling true
    m_bMouseScroll = m_bKeyboardScroll = true;

    // set default tilesize
    m_iTILEWIDTH = 60;
    m_iTILEHEIGHT = 30;

    // set offset to 0
    m_fOffsetX = 0.0f;
    m_fOffsetY = 0.0f;

    // the viewport is by default as big as the frame
    m_ViewPort.x = 0;
    m_ViewPort.y = 0;
    m_ViewPort.w = m_pSDLFrame->getWidth();
    m_ViewPort.h = m_pSDLFrame->getHeight();

    // set the default speed for mapscrolling to 2.5f
    m_fScrollSpeed = 2.5f;

    // set title of sdl frame
    m_pSDLFrame->setTitle( "Herr der Trme v 0.1" );

    // read in the config file for the engine if one is given
    ifstream ifs;
    // open filestream
    ifs.open( file.c_str(), ios::in );

    string read;

    // read until eof
    while ( ifs )
    {
        // read
        ifs >> read;

        // e.g. (viewport) 0 0 640 480
        if ( read == "(viewport)" )
        {
            ifs >> m_ViewPort.x;
            ifs >> m_ViewPort.y;
            ifs >> m_ViewPort.w;
            ifs >> m_ViewPort.h;
        }
        // e.g. (startpos) 200 200
        else if( read == "(startpos)" )
        {
            ifs >> m_fOffsetX;
            ifs >> m_fOffsetY;
        }
        // e.g. (scrollspeed) 5.0
        else if( read == "(scrollspeed)" )
        {
            ifs >> m_fScrollSpeed;
        }
        // e.g. (mousescroll) 1
        else if( read == "(mousescroll)" )
        {
            ifs >> m_bMouseScroll;
        }
        // e.g. (keyboardscroll) 0
        else if( read == "(keyboardscroll)" )
        {
            ifs >> m_bKeyboardScroll;
        }
        // reset readstring
        read = "";
        }

    // close the filestream
    ifs.close();

    // load graphics out of the graphics config
    if( !loadGraphics( "system/graphics.cfg" ) )
    {
        cout << "| !! CIsoEngine _ Couldn't load graphics!\n";
        m_pLog->write( "| !! CIsoEngine _ Couldn't load graphics!" );
    }
    else
    {
        cout << "|    CIsoEngine _ Successfully loaded graphics!\n";
        m_pLog->write( "|    CIsoEngine _ Successfully loaded graphics!" );
    }

    loadMap( "maps/test.map" );

    // create a new map !TEMPORARY!
    //createMap( 25, 25, 0, 1 );
    // save the map !TEMPORARY!
    //saveMap( "maps/test.map" );

    // set fHalMapWidth, used for drawing the view correctly and getting the mousecoords
    m_fHalfMapWidth = ( m_ViewPort.w / 2 ) + m_ViewPort.x;

    
    // set bounding box for map used for scrolling
    m_MapBounds.x = (int)( m_fHalfMapWidth + m_iTILEHEIGHT + ( m_iMapWidth / 2 * m_iTILEWIDTH) * -1 );
    m_MapBounds.y = (int)( m_ViewPort.y );
    m_MapBounds.w = (int)( ( m_iMapWidth * m_iTILEWIDTH ) - m_ViewPort.w );
    m_MapBounds.h = (int)( ( m_iMapHeight * m_iTILEHEIGHT ) - m_ViewPort.h );
    
    // report to log that we finished init
    cout << "|    CIsoEngine _ Init was successfull\n";
    m_pLog->write( "|    CIsoEngine _ Init was successfull" );

    // return true on success
    return true;
}

//-----------------------------------------------------------------------------
void CIsoEngine::setSDLFrame( CSDL *sdlFrame )
{
    m_pSDLFrame = sdlFrame;
}

//-----------------------------------------------------------------------------
/*
    function to load graphics out of a groupfile, called in init()
    groundgroups will be stored in "ground" array
    detailgroups in "detail" array
    e.g.
    [groundgroups] 2 60 30
    (groundgroup) grass gfx/ground/grass.png 5
    (groundgroup) stone gfx/ground/grass.png 4
*/
bool CIsoEngine::loadGraphics( string file )
{
    int actGroundGroup = 0;
    int actDetailGroup = 0;
    int width = 0;
    int height = 0;

    // read in the config file for the graphics
    ifstream ifs;
    // open filestream
    ifs.open( file.c_str(), ios::in );

    // if file doesn't exist
    if( !ifs )
    {
        cout << "| !! CIsoEngine _ Couldn't open graphicfile: " << file << "\n";
        m_pLog->write( "| !! CIsoEngine _ Couldn't open graphicfile: " + file );
        return false;
    }

    string read;
    string groupName;
    string groupFile;
    int anzahl;

    SDL_Surface* tiles = NULL;

    while ( ifs )
    {
        ifs >> read;

        if( read == "[groundgroups]" )
        {
            ifs >> m_iNumberGround;

            cout << "There are " << m_iNumberGround << " group/s for ground in file\n";

            ifs >> m_iTILEWIDTH;
            ifs >> m_iTILEHEIGHT;

            cout << "IsoEngine groundsize is set to " << m_iTILEWIDTH << "x" << m_iTILEHEIGHT << "\n";

            m_pGround = new CGroup[ m_iNumberGround ];
        }

        else if( read == "[detailgroups]" )
        {
            ifs >> m_iNumberDetail;

            cout << "There are " << m_iNumberDetail << " group/s for detail in file\n";

            m_pDetail = new CGroup[ m_iNumberDetail ];
        }

        else if( read == "(groundgroup)"  )
        {
            ifs >> groupName;
            ifs >> groupFile;
            ifs >> anzahl;

            tiles = m_pSDLFrame->loadImage( groupFile );

            m_pGround[ actGroundGroup ].init( groupName, tiles, anzahl, m_iTILEWIDTH, m_iTILEHEIGHT );
            ++actGroundGroup;
            cout << "Groundgroup " << groupName << " with " << groupFile << " as Image with " << anzahl << " Tiles loaded\n";
        }
        else if( read == "(detailgroup)" )
        {
            ifs >> groupName;
            ifs >> groupFile;
            ifs >> anzahl;
            ifs >> width;
            ifs >> height;

            tiles = m_pSDLFrame->loadImage( groupFile );

            m_pDetail[ actDetailGroup ].init( groupName, tiles, anzahl, width, height );
            ++actDetailGroup;
            cout << "Detailgroup " << groupName << " with " << groupFile << " as Image with " << anzahl << " Tiles loaded\n";
        }
        read = "";
    }
    ifs.close();

    return true;
}

//-----------------------------------------------------------------------------
void CIsoEngine::drawTiles()
{
    float fX = 0;
    float fY = 0;

    SDL_Rect tileRect = { 0, 0, m_iTILEWIDTH, m_iTILEHEIGHT };

    // draw tiles
    for( int y = 0; y < m_iMapHeight; ++y)
    {
        for( int x = 0; x < m_iMapWidth; ++x )
        {
            fX = ( ( x - y ) * ( m_iTILEWIDTH / 2 ) ) + m_fHalfMapWidth;
            fY = ( ( x + y ) * ( m_iTILEHEIGHT / 2 ) );
            SDL_Rect destRect =
            {
                (int)( fX - m_fOffsetX ),
                (int)( fY - m_fOffsetY ),
                m_iTILEWIDTH,
                m_iTILEHEIGHT
            };
            
            tileRect.h = m_iTILEHEIGHT;

            // test if the image we are going to draw is in our viewPort
            if( clip( m_ViewPort, tileRect, destRect ) )
            {
                // set the source rect
                SDL_Rect srcRect;
                srcRect.x = m_pMap[ x + y * m_iMapWidth ].m_iGroundTile * m_pGround[ m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup ].getWidth();
                srcRect.y = 0;
                srcRect.w = m_pGround[ m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup ].getWidth();
                srcRect.h = m_pGround[ m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup ].getHeight();

                // blit the groundtile
                m_pSDLFrame->blitSurface( m_pGround[m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup ].getTiles(), srcRect, destRect );
            }

            // check if there are any detail/overlay graphics loaded
            if( m_iNumberDetail )
            {
                // calculate the new destination y, because detailtiles can be higher than TILEWIDTHxTILEHEIGHT
                destRect.y -= ( m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getHeight() - m_iTILEHEIGHT );
                // set new tileRect height, because detailtiles can be higher than TILEWIDTHxTILEHEIGHT
                tileRect.h = m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getHeight();

                // test if the image we are going to draw is in our viewPort
                if( clip( m_ViewPort, tileRect, destRect ) )
                {
                    // set the source rect
                    SDL_Rect srcRect;
                    srcRect.x = m_pMap[ x + y * m_iMapWidth ].m_iDetailTile * m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getWidth();
                    srcRect.y = 0;
                    srcRect.w = m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getWidth();
                    srcRect.h = m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getHeight();

                    // blit the detailtile
                    m_pSDLFrame->blitSurface( m_pDetail[ m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup ].getTiles(), srcRect, destRect );
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------
bool CIsoEngine::idle( unsigned long frameTime )
{
    m_dwFrameTime = frameTime;

    // if mouse scrolling is true
    if( m_bMouseScroll )
    {
        // scroll left
        if( m_pSDLFrame->getMouseX() < m_ViewPort.x + 20 )
            scroll ( west );
        // scroll right
        else if( m_pSDLFrame->getMouseX() > m_ViewPort.x + m_ViewPort.w - 20 )
            scroll ( east );
        // scroll up
        if( m_pSDLFrame->getMouseY() < m_ViewPort.y + 20 )
            scroll ( north );
        // scroll down
        else if( m_pSDLFrame->getMouseY() > m_ViewPort.y + m_ViewPort.h - 20 )
            scroll ( south );
    }
    
    // if keyboard scrolling is true
    if( m_bKeyboardScroll )
    {
        // scroll left
        if( m_pSDLFrame->getKeys()[ SDLK_LEFT ] )
            scroll ( west );
        // scroll right
        else if( m_pSDLFrame->getKeys()[ SDLK_RIGHT ] )
            scroll ( east );
        // scroll up
        if( m_pSDLFrame->getKeys()[ SDLK_UP ] )
            scroll ( north );
        // scroll down
        else if( m_pSDLFrame->getKeys()[ SDLK_DOWN ] )
            scroll ( south );
    }

    // fill background
    SDL_Rect screen = { m_ViewPort.x, m_ViewPort.y, m_ViewPort.w, m_ViewPort.h };
    SDL_FillRect( m_pSDLFrame->getVideomem(), &screen, 0x000000 );

    // draw the tiles
    drawTiles();

    return true;
}

//-----------------------------------------------------------------------------
/*
	used for testing if the given rect is in the viewPort
*/
bool CIsoEngine::clip( SDL_Rect viewPort, SDL_Rect srcRect, SDL_Rect dstRect )
{
    if(    dstRect.x < viewPort.x + viewPort.w && viewPort.x < dstRect.x + srcRect.w
            && dstRect.y < viewPort.y + viewPort.h && viewPort.y < dstRect.y + srcRect.h )
        return true;
    // dstRect isn't in the viewPort
    return false;
}

//-----------------------------------------------------------------------------
void CIsoEngine::scroll( direction d )
{
    switch( d )
    {
        case north:
            if( m_fOffsetY > m_MapBounds.y )
                m_fOffsetY -=  ( (float)m_dwFrameTime / 10.0f ) * ( m_fScrollSpeed / 2 );
            break;
            
        case east:
            if( m_fOffsetX < m_MapBounds.x + m_MapBounds.w )
                m_fOffsetX +=  ( (float)m_dwFrameTime / 10.0f ) * m_fScrollSpeed;
            break;
            
        case south:
            if( m_fOffsetY < m_MapBounds.y + m_MapBounds.h )
                m_fOffsetY +=  ( (float)m_dwFrameTime / 10.0f ) * ( m_fScrollSpeed / 2 );
            break;

        case west:
            if( m_fOffsetX > m_MapBounds.x )
                m_fOffsetX -=  ( (float)m_dwFrameTime / 10.0f ) * m_fScrollSpeed;
            break;

        default:
            break;
    }
}

//-----------------------------------------------------------------------------
/*
	maploading routine loadMap( "blafasel.map" );
*/
void CIsoEngine::loadMap( string file )
{
    // open filereading stream
    ifstream ifs;
    ifs.open( file.c_str(), ios::in );

    // if file exists
    if( ifs )
    {
        // load width and map of the height
        ifs >> m_iMapWidth;
        ifs >> m_iMapHeight;
        // maybe mapname?

        // create new map with the given width and height
        createMap( m_iMapWidth, m_iMapHeight, 0, 0 );

        // load the mapdata
        for( int x = 0; x < m_iMapWidth; ++x )
        {
            for( int y = 0; y < m_iMapHeight; ++y )
            {
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup;
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iGroundTile;
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup;
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iDetailTile;
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iCollision;
                ifs >> m_pMap[ x + y * m_iMapWidth ].m_iEntityID;
            }
        }
    }
}

//-----------------------------------------------------------------------------
/*
	mapsaving routine saveMap( "blafasel.map" );
*/
void CIsoEngine::saveMap( string file )
{
    // open filewriting stream
    ofstream ofs;
    ofs.open( file.c_str(), ios::out );

    // save width and height of the map
    ofs << m_iMapWidth << " ";
    ofs << m_iMapHeight << " ";
    // maybe mapname?

    // save the groups and tiles
    for( int x = 0; x < m_iMapWidth; ++x )
    {
        for( int y = 0; y < m_iMapHeight; ++y )
        {
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup << " ";
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iGroundTile << " ";
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup << " ";
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iDetailTile << " ";
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iCollision << " ";
            ofs << m_pMap[ x + y * m_iMapWidth ].m_iEntityID << " ";
        }
    }
}

//-----------------------------------------------------------------------------
/*
	create new map like createMap( 50, 50, 0, 1 );
*/
void CIsoEngine::createMap( int width, int height, int group, int tile )
{
    // set width and height of the map
    m_iMapWidth = width;
    m_iMapHeight = height;

    // allocate the memory we need
    m_pMap = new CMap[ m_iMapWidth * m_iMapHeight ];

    // set the groups and tiles
    for( int x = 0; x < m_iMapWidth; ++x )
    {
        for( int y = 0; y < m_iMapHeight; ++y )
        {
            m_pMap[ x + y * m_iMapWidth ].m_iGroundGroup = group;
            m_pMap[ x + y * m_iMapWidth ].m_iGroundTile = tile;
            m_pMap[ x + y * m_iMapWidth ].m_iDetailGroup = 0;
            m_pMap[ x + y * m_iMapWidth ].m_iDetailTile = 0;
            m_pMap[ x + y * m_iMapWidth ].m_iCollision = 0;
            m_pMap[ x + y * m_iMapWidth ].m_iEntityID = 0;
        }
    }
}

//-----------------------------------------------------------------------------
void CIsoEngine::setMouseScroll( bool b )
{
    m_bMouseScroll = b;
}

//-----------------------------------------------------------------------------
void CIsoEngine::setKeyboardScroll( bool b )
{
    m_bKeyboardScroll = b;
}
