//-----------------------------------------------------------------------------
/*
    CIsoEngine.h - Author: Marcus Krner - 2003 - This source is GPL
    isometric engine class ( needs a CSDL and a CLog object )
*/

#ifndef CISOENGINE_H_
#define CISOENGINE_H_

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <iostream>
#include <string>

#include "CSDL.h"
#include "CGroup.h"
#include "CMap.h"
#include "CLog.h"
#include "globals.h"

using namespace std;


//-----------------------------------------------------------------------------
/*
    class CIsoEngine
*/
class CIsoEngine
{
    CSDL* m_pSDLFrame;                     // CSDL Frame object
    CLog* m_pLog;                                  // our logging class

    SDL_Rect m_ViewPort;
    SDL_Rect m_MapBounds;                // store the rect around the diamond map

    CMap* m_pMap;                               // store our mapdata

    CGroup* m_pGround;                      // store groundtile
    CGroup* m_pDetail;                         // store detailtiles

    int m_iNumberGround;                    // stores the number of ground groups
    int m_iNumberDetail;                       // stores the number of detail groups
    int m_iMapWidth;
    int m_iMapHeight;
    int m_iTILEHEIGHT;                         // height of the groundtile
    int m_iTILEWIDTH;                           // width of the groundtile

    float m_fOffsetX;                               // OffsetX used for scrolling and placing entities
    float m_fOffsetY;                               // OffsetY used for scrolling and placing entities
    float m_fHalfMapWidth;
    float m_fScrollSpeed;

    bool m_bMouseScroll;
    bool m_bKeyboardScroll;

    unsigned long m_dwFrameTime;
    
public:
    CIsoEngine();
    ~CIsoEngine();

    CSDL* getSDLFrame(){ return m_pSDLFrame; };

    SDL_Rect getViewPort(){ return m_ViewPort; };

    CMap* getMap(){ return m_pMap; };

    CGroup *getGround(){ return m_pGround; };
    CGroup *getDetail(){ return m_pDetail; };

    void createMap( int width, int height, int group, int tile );
    void loadMap( string file );
    void saveMap( string file );
    void setSDLFrame( CSDL *sdlFrame );
    void scroll( direction d );
    void setMouseScroll( bool b );
    void setKeyboardScroll( bool b );
    void drawTiles();

    int getNumberGround(){ return m_iNumberGround; };
    int getNumberDetail(){ return m_iNumberDetail; };
    int getMapWidth(){ return m_iMapWidth; };
    int getMapHeight(){ return m_iMapHeight; };
    int getTileHeight(){ return m_iTILEHEIGHT; };
    int getTileWidth(){ return m_iTILEWIDTH; };


    bool getKeyboardScroll(){ return m_bKeyboardScroll; };
    bool clip( SDL_Rect viewPort, SDL_Rect srcRect, SDL_Rect dstRect );
    bool idle( unsigned long frameTime );
    bool init( CSDL *sdlFrame, CLog *log, string file );
    bool loadGraphics( string file );

    float getOffsetX(){ return m_fOffsetX; };
    float getOffsetY(){ return m_fOffsetY; };
    float getHalfMapWidth(){ return m_fHalfMapWidth; };
};

#endif
