//-----------------------------------------------------------------------------
/*
    CSDL.h - Author: Marcus Krner - 2003 - This source is GPL
    simple SDL wrapper class
*/

#include "CSDL.h"

//-----------------------------------------------------------------------------
CSDL::CSDL() : m_pVideomem( NULL ), m_pLog( NULL )
{
}

//-----------------------------------------------------------------------------
void CSDL::init( CLog* log )
{
    m_iLastkey = 0;
    m_pLog = log;
    m_pLog->write( "|" );
}

//-----------------------------------------------------------------------------
void CSDL::shutdown()
{
    SDL_Quit();
    delete m_pVideomem;
}

//-----------------------------------------------------------------------------
CSDL::~CSDL()
{
}

//-----------------------------------------------------------------------------
int CSDL::setFrame( int width, int height, int bpp, Uint32 flags )
{
    // init SDL_VIDEO
    if ( !initSDL( SDL_INIT_VIDEO ) )
    {
        cout << "| !! CSDL _Couldn't initialze SDL video\n";
        m_pLog->write( "| !! CSDL _ Couldn't initialze SDL video");
        return 0;
    }
    else
    {
        cout << "|    CSDL _ SDL video initialzed\n";
        m_pLog->write( "|    CSDL _ SDL video initialzed" );
    }

    m_iWidth = width;
    m_iHeight = height;

    // set video mode
    if( !setVideoMode( m_iWidth, m_iHeight, bpp, flags ) )
    {
        cout << "| !! CSDL _ Couldn't set video mode: " << m_iWidth << "x" << m_iHeight << " @ " << bpp << "\n";
        m_pLog->write( "| !! CSDL _ Couldn't set video mode" ); //: " + iWidth + "x" + iHeight + " @ " + bpp + "\n" );
        return 0;
    }
    else
    {
        cout << "|    CSDL _ Video mode set successfully to: " << m_iWidth << "x" << m_iHeight << " @ " << bpp << "\n";
        m_pLog->write( "|    CSDL _ Video mode set successfully" ); // to: " + ostrstream( iWidth ) + "x" + ostrstream( iHeight ) + " @ " + ostrstream( bpp ) + "\n" );
    }
    return 1;
}

int CSDL::setFrame( string file )
{
    m_iMouseClickX = m_iMouseClickY = 0;
    m_iMouseX = m_iMouseY = 30;

    // read from our configfile
    ifstream ifs;
    ifs.open( file.c_str(), ios::in );

    if( !ifs )
    {
        cout << "| !! CSDL _ Couldn't read configfile: " << file << "\n";
        m_pLog->write( "| !! CSDL _ Couldn't read configfile: " + file );
        m_iWidth = 640;
        m_iHeight = 480;
        setFrame( m_iWidth, m_iHeight, 0, SDL_HWSURFACE | SDL_DOUBLEBUF );
    }

    string read;

    while ( ifs )
    {
        ifs >> read;

        if ( read == "(screen)" )
        {
            int width, height, depth, full;

            ifs >> m_iWidth;
            ifs >> m_iHeight;
            ifs >> depth;
            ifs >> full;

            if( full )
                setFrame( m_iWidth, m_iHeight, depth, SDL_SWSURFACE | SDL_DOUBLEBUF | SDL_FULLSCREEN );
            else
                setFrame( m_iWidth, m_iHeight, depth, SDL_SWSURFACE | SDL_DOUBLEBUF ) ;
        }
        // font?
        // else if ( read == "(font)" )
        // {
        // }
    }
    m_pKeys = SDL_GetKeyState( NULL );
    ifs.close();
    return 1;
}

//-----------------------------------------------------------------------------
void CSDL::flip()
{
    SDL_Flip( SDL_GetVideoSurface() );
}

//-----------------------------------------------------------------------------
bool	CSDL::initSDL( Uint32 flags )
{
    if( SDL_Init( flags ) < 0 )
        return false;
    return true;
}

//-----------------------------------------------------------------------------
bool CSDL::setVideoMode( int width, int height, int bpp, Uint32 flags )
{
    m_iWidth = width;
    m_iHeight = height;
    m_pVideomem = SDL_SetVideoMode( m_iWidth, m_iHeight, bpp, flags );
    if( m_pVideomem == NULL )
        return false;
    setKeyRepeat( true );
    return true;
}

//-----------------------------------------------------------------------------
void CSDL::setKeyRepeat( bool value )
{
    if( value )
        SDL_EnableKeyRepeat( SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL );
    else
        SDL_EnableKeyRepeat( 0, 0);
}

//-----------------------------------------------------------------------------
void CSDL::setTitle( string title )
{
    SDL_WM_SetCaption( title.c_str(), NULL );
}

//-----------------------------------------------------------------------------
bool CSDL::handleEvents()
{
    while(SDL_PollEvent(&m_Event))
    {
        switch( m_Event.type )
        {
            case SDL_QUIT:
                return false;
                break;
                
            case SDL_KEYDOWN:
                if( m_Event.key.keysym.sym == SDLK_ESCAPE)
                    return false;
                break;

            case SDL_MOUSEMOTION:
                m_iMouseX = m_Event.motion.x;
                m_iMouseY = m_Event.motion.y;
                break;
            case SDL_MOUSEBUTTONDOWN:
                m_iMouseClickX = m_Event.button.x;
                m_iMouseClickY = m_Event.button.y;
        }
        m_pKeys = SDL_GetKeyState( NULL );
    }
    m_iMouseState = SDL_GetMouseState( &m_iMouseClickX, &m_iMouseClickY );
    return true;
}

//-----------------------------------------------------------------------------
void CSDL::blitSurface( SDL_Surface *surface, SDL_Rect pos)
{
    SDL_BlitSurface( surface, 0, m_pVideomem, &pos);
}

//-----------------------------------------------------------------------------
void CSDL::blitSurface( SDL_Surface *surface, float x, float y )
{
    SDL_Rect pos;
    pos.x = (int)x;
    pos.y = (int)y;
    SDL_BlitSurface( surface, 0, m_pVideomem, &pos);
}

//-----------------------------------------------------------------------------
void CSDL::blitSurface( SDL_Surface *surface, SDL_Rect srcRect, SDL_Rect destRect )
{
    SDL_BlitSurface( surface, &srcRect, m_pVideomem, &destRect);
}

//-----------------------------------------------------------------------------
void CSDL::blitSurface( SDL_Surface *surface, int x, int y )
{
    SDL_Rect pos;
    pos.x = x;
    pos.y = y;
    SDL_BlitSurface( surface, 0, m_pVideomem, &pos);
}

//-----------------------------------------------------------------------------
bool CSDL::keyPressed( int key )
{
    if( getKeys() [ key ] && m_iLastkey != key )
    {
        m_iLastkey = key;
        return true;
    }
    else if(!this->getKeys() [ key ] && m_iLastkey == key)
    {
        m_iLastkey = 0;
        return false;
    }
    return false;
}

//-----------------------------------------------------------------------------
SDL_Surface* CSDL::loadImage( string file )
{
    // load the image
    SDL_Surface *tmp = IMG_Load( file.c_str() );

    // check if imageloading went fine
    if( tmp == NULL )
    {
        cout << "| !! CSDL _ Couldn't load image " << file << "\n";
        m_pLog->write( "| !! CSDL _ Couldn't load image " + file );
        return 0;
    }

    // Colorkey our surface with 0xFF00FF
    SDL_SetColorKey( tmp, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB( tmp->format, 255, 0, 255) );

    // return the image in DisplayFormat ( e.g. 8bit, 16bit, 32bit... )
    return SDL_DisplayFormat( tmp );
}

//-----------------------------------------------------------------------------
Uint32 CSDL::getPixel( SDL_Surface *surface, int x, int y )
{
    int bpp = surface->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to retrieve */
    Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       /* shouldn't happen, but avoids warnings */
    }
}

//-----------------------------------------------------------------------------
void CSDL::setPixel( int x, int y, Uint32 pixel )
{
    if( x >= 0 && y >= 0 && x < m_iWidth && y < m_iHeight )
    {
    int bpp = m_pVideomem->format->BytesPerPixel;
    /* Here p is the address to the pixel we want to set */
    Uint8 *p = (Uint8 *)m_pVideomem->pixels + y * m_pVideomem->pitch + x * bpp;

    switch(bpp) {
    case 1:
        *p = pixel;
        break;

    case 2:
        *(Uint16 *)p = pixel;
        break;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN) {
            p[0] = (pixel >> 16) & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = pixel & 0xff;
        } else {
            p[0] = pixel & 0xff;
            p[1] = (pixel >> 8) & 0xff;
            p[2] = (pixel >> 16) & 0xff;
        }
        break;

    case 4:
        *(Uint32 *)p = pixel;
        break;
    }
    }
}
