//-----------------------------------------------------------------------------
/*
    CSDL.h - Author: Marcus Krner - 2003 - This source is GPL
    simple SDL wrapper class
*/

#ifndef CSDL_H_
#define CSDL_H_

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <string>
#include <iostream>
#include <fstream>

#include "CLog.h"

using namespace std;

//-----------------------------------------------------------------------------
class CSDL
{
    CLog* m_pLog;
    
    SDL_Surface* m_pVideomem;

    Uint8* m_pKeys;
    Uint8 m_iMouseState;

    SDL_Event m_Event;

    int m_iMouseX, m_iMouseY;
    int m_iMouseClickX,  m_iMouseClickY;
    int m_iWidth, m_iHeight;
    int m_iLastkey;

public:
    CSDL();
    ~CSDL();

    Uint32 getPixel( SDL_Surface *surface, int x, int y );
    
    SDL_Surface* loadImage( std::string file );
    SDL_Surface* getVideomem() { return m_pVideomem; };

    SDL_Event getEvent() { return m_Event; };

    Uint8* getKeys() { return m_pKeys; };
    Uint8 getMouseState() { return m_iMouseState; };

    void blitSurface( SDL_Surface *surface, SDL_Rect pos);
    void blitSurface( SDL_Surface *surface, SDL_Rect srcRect, SDL_Rect destRect );
    void blitSurface( SDL_Surface *surface, float x, float y );
    void blitSurface( SDL_Surface *surface, int x, int y );
    void setPixel( int x, int y, Uint32 pixel );
    void setTitle( string title );
    void setKeyRepeat( bool value );
    void flip();
    void shutdown();
    void init( CLog *log );

    bool initSDL(Uint32 flags);
    bool setVideoMode(int width, int height, int bpp, Uint32 flags);
    bool handleEvents();
    bool keyPressed( int key );

    int setFrame( int width, int height, int bpp, Uint32 flags );
    int setFrame( string file );
    int round( double f ) { return (int)((f>0)?(f+0.5):(f-0.5)); }

    // accessors
    int getMouseX(){ return m_iMouseX; };
    int getMouseY(){ return m_iMouseY; };
    int getMouseXClick(){ return m_iMouseClickX; };
    int getMouseYClick(){ return m_iMouseClickY; };
    int getWidth(){ return m_iWidth; };
    int getHeight(){ return m_iHeight; };
};

#endif
