//-----------------------------------------------------------------------------
/*
    menu.cpp - Author: Marcus Krner - 2003 - This source is GPL
*/

#include "menu.h"

Menu::Menu( CSimpleFont *font )
{
  m_pMenubar = NULL;
  m_pEssence = NULL;
  m_pEnergy = NULL;
  m_pSelectedChar = NULL;
  m_pFont = font;
}

Menu::~Menu()
{
  SDL_FreeSurface( m_pMenubar );
  SDL_FreeSurface( m_pEssence );
  SDL_FreeSurface( m_pEnergy );
}

void Menu::loadMenu( std::string menuConfig, CSDL *sdlFrame )
{
  // read in the config file for the graphics
  std::ifstream ifs;
  // open filestream
  ifs.open( menuConfig.c_str(), std::ios::in );

  // if file doesn't exist
  if( !ifs )
  {
        cout << "| !! Menu _ Couldn't open menuConfig: " << menuConfig << "\n";
        return;
  }

  std::string read;
  
  while ( ifs )
  {
    ifs >> read;

    if( read == "(menubar)" )
    {
      ifs >> read;

      m_pMenubar = sdlFrame->loadImage( read );

      ifs >> m_iMenubarX;
      ifs >> m_iMenubarY;
    }
    else if( read == "(essence)" )
    {
      ifs >> read;

      m_pEssence = sdlFrame->loadImage( read );

      ifs >> m_iEssenceX;
      ifs >> m_iEssenceY;
    }
    else if( read == "(energy)" )
    {
      ifs >> read;

      m_pEnergy = sdlFrame->loadImage( read );

      ifs >> m_iEnergyX;
      ifs >> m_iEnergyY;
    }

    read = "";
  }

}

void Menu::draw( CSDL *sdlFrame )
{
  sdlFrame->blitSurface( m_pMenubar, m_iMenubarX, m_iMenubarY );
  sdlFrame->blitSurface( m_pEssence, m_iEssenceX, m_iEssenceY );
  sdlFrame->blitSurface( m_pEnergy, m_iEnergyX, m_iEnergyY );
  
  if( m_pSelectedChar )
  {
    m_pFont->putString( m_pSelectedChar->getName(), 295, 470 );

    // draw Attackbuttons
    int startx = 320 - m_pSelectedChar->getNumberAttacks() * 15;
    int i = 0;
    Attack *tmp = m_pSelectedChar->getAttacks();
    while( tmp->getNext() != NULL )
    {
      tmp = tmp->getNext();
      sdlFrame->blitSurface( tmp->getMenuPic(), startx + i * 30, 435 );
      ++i;
    }
  }
}

void Menu::idle( Char *c )
{
  m_pSelectedChar = c;
}
