/*
    particeleffect.cpp - Author: Marcus Krner - 2003 - This source is GPL
*/

#include "imageparticleeffect.h"

ImageParticleEffect::ImageParticleEffect( SDL_Surface *image, float x, float y, double angle, float speed, float particleangle, float particlespeed, int width, int maxParticles, int particleLifeTime )
{
  m_pImage = image;
  m_iWidth = width;
  m_fX = x;
  m_fY = y;
  m_fAngle = angle;
  m_fSpeed = speed;
  m_fParticleAngle = particleangle;
  m_fParticleSpeed = particlespeed;
  m_iMaxParticles = maxParticles;
  m_iActParticles = 0;
  m_iParticleLifeTime = particleLifeTime;
  m_pParticles = new ImageParticle();    
}

ImageParticleEffect::~ImageParticleEffect()
{
  ImageParticle *tmp = m_pParticles;
  while( tmp->getNext() )
    tmp = tmp->getNext();

  while( tmp->getPrev() )
  {
    ImageParticle *prev = tmp->getPrev();
    delete tmp;
    tmp = prev;    
  }
}

                       
void ImageParticleEffect::draw( CSDL *sdlFrame )
{
  ImageParticle *tmp = m_pParticles;
  while( tmp->getNext() )
  {    
    tmp = tmp->getNext();
    tmp->draw( sdlFrame );
  }
}

void ImageParticleEffect::move( unsigned long frameTime )
{
  // add particle if maxParticles isn't reached
  if( m_iActParticles < m_iMaxParticles )
    addParticle();

  // move particle effect
    m_fX += ( cos( ( M_PI * m_fAngle ) / 180.0f) * m_fSpeed ) * static_cast<float>( frameTime ) / 1000.0f;
    m_fY += ( sin( ( M_PI * m_fAngle ) / 180.0f) * m_fSpeed ) * static_cast<float>( frameTime ) / 1000.0f;
    
  // get first particle in list
  ImageParticle *tmp = m_pParticles->getNext();
  while( tmp )
  {
    // p points at current particle
    ImageParticle *p = tmp;
    // save the next particle in tmp
    tmp = tmp->getNext();
    // move the current particle
    if( !p->move( frameTime ) )
    {
      // delete the current particle if move returns false
      delParticle( p );
    }
  }
}

void ImageParticleEffect::delParticle( ImageParticle *p )
{
  if( p->getNext() )
  {
    if( p->getPrev() )
    {
      p->getPrev()->setNext( p->getNext() );
      p->getNext()->setPrev( p->getPrev() );
      delete p;
    }
    else
    {
      p->getNext()->setPrev( NULL );
      delete p;
    }
  }
  else
  {
    p->getPrev()->setNext( NULL );
    delete p;
  }
  --m_iActParticles;  
}

void ImageParticleEffect::addParticle()
{
  srand( SDL_GetTicks() );
  ImageParticle *tmp = m_pParticles;
  while( tmp->getNext() )
    tmp = tmp->getNext();
  tmp->setNext( new ImageParticle( m_pImage, m_fX, m_fY,  cos( ( M_PI * m_fParticleAngle ) / 180.0f ) * m_fParticleSpeed - ( m_iWidth / 2.0f ) +  0.01f * static_cast<float>( rand()%( m_iWidth * 100 ) ), sin( ( M_PI * m_fParticleAngle ) / 180.0f ) * m_fParticleSpeed -  ( m_iWidth / 2.0f ) +  0.01f * static_cast<float>( rand()%( m_iWidth * 100 ) ), m_iParticleLifeTime +  rand()%20 ) );
  tmp->getNext()->setPrev( tmp );
  ++m_iActParticles;
}
